@@ -1,33 +1,6 @@
-#!/usr/bin/env python
-# $Id: __main__.py 1262 2014-04-11 22:13:10Z g.rodola $
-
-#  ======================================================================
-#  Copyright (C) 2007-2014 Giampaolo Rodola' <g.rodola@gmail.com>
-#
-#                         All Rights Reserved
-#
-# Permission is hereby granted, free of charge, to any person
-# obtaining a copy of this software and associated documentation
-# files (the "Software"), to deal in the Software without
-# restriction, including without limitation the rights to use,
-# copy, modify, merge, publish, distribute, sublicense, and/or sell
-# copies of the Software, and to permit persons to whom the
-# Software is furnished to do so, subject to the following
-# conditions:
-#
-# The above copyright notice and this permission notice shall be
-# included in all copies or substantial portions of the Software.
-#
-# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
-# EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
-# OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
-# NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
-# HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
-# WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
-# FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
-# OTHER DEALINGS IN THE SOFTWARE.
-#
-#  ======================================================================
+# Copyright (C) 2007 Giampaolo Rodola' <g.rodola@gmail.com>.
+# Use of this source code is governed by MIT license that can be
+# found in the LICENSE file.
 
 """
 Start a stand alone anonymous FTP server from the command line as in:
@@ -35,19 +8,22 @@ Start a stand alone anonymous FTP server from the command line as in:
 $ python -m pyftpdlib
 """
 
+import logging
 import optparse
-import sys
 import os
+import sys
 
-from pyftpdlib import __ver__
-from pyftpdlib.authorizers import DummyAuthorizer
-from pyftpdlib.handlers import FTPHandler
-from pyftpdlib.servers import FTPServer
-from pyftpdlib._compat import getcwdu
+from . import __ver__
+from ._compat import getcwdu
+from .authorizers import DummyAuthorizer
+from .handlers import FTPHandler
+from .log import config_logging
+from .servers import FTPServer
 
 
 class CustomizedOptionFormatter(optparse.IndentedHelpFormatter):
     """Formats options shown in help in a prettier way."""
+
     def format_option(self, option):
         result = []
         opts = self.option_strings[option]
@@ -60,7 +36,7 @@ class CustomizedOptionFormatter(optparse.IndentedHelpFormatter):
 
 def main():
     """Start a stand alone anonymous FTP server."""
-    usage = "python -m pyftpdlib.ftpserver [options]"
+    usage = "python -m pyftpdlib [options]"
     parser = optparse.OptionParser(usage=usage, description=main.__doc__,
                                    formatter=CustomizedOptionFormatter())
     parser.add_option('-i', '--interface', default=None, metavar="ADDRESS",
@@ -69,7 +45,7 @@ def main():
     parser.add_option('-p', '--port', type="int", default=2121, metavar="PORT",
                       help="specify port number to run on (default 2121)")
     parser.add_option('-w', '--write', action="store_true", default=False,
-                      help="grants write access for the anonymous user "
+                      help="grants write access for logged in user "
                            "(default read-only)")
     parser.add_option('-d', '--directory', default=getcwdu(), metavar="FOLDER",
                       help="specify the directory to share (default current "
@@ -79,18 +55,25 @@ def main():
     parser.add_option('-r', '--range', default=None, metavar="FROM-TO",
                       help="the range of TCP ports to use for passive "
                            "connections (e.g. -r 8000-9000)")
+    parser.add_option('-D', '--debug', action='store_true',
+                      help="enable DEBUG logging level")
     parser.add_option('-v', '--version', action='store_true',
                       help="print pyftpdlib version and exit")
     parser.add_option('-V', '--verbose', action='store_true',
                       help="activate a more verbose logging")
+    parser.add_option('-u', '--username', type=str, default=None,
+                      help="specify username to login with (anonymous login "
+                           "will be disabled and password required "
+                           "if supplied)")
+    parser.add_option('-P', '--password', type=str, default=None,
+                      help="specify a password to login with (username "
+                           "required to be useful)")
 
     options, args = parser.parse_args()
     if options.version:
         sys.exit("pyftpdlib %s" % __ver__)
-    if options.verbose:
-        import logging
-        import pyftpdlib.log
-        pyftpdlib.log.LEVEL = logging.DEBUG
+    if options.debug:
+        config_logging(level=logging.DEBUG)
 
     passive_ports = None
     if options.range:
@@ -109,17 +92,31 @@ def main():
         options.interface = '0.0.0.0'
 
     authorizer = DummyAuthorizer()
-    perm = options.write and "elradfmwM" or "elr"
-    authorizer.add_anonymous(options.directory, perm=perm)
+    perm = options.write and "elradfmwMT" or "elr"
+    if options.username:
+        if not options.password:
+            parser.error(
+                "if username (-u) is supplied, password ('-P') is required")
+        authorizer.add_user(options.username,
+                            options.password,
+                            options.directory,
+                            perm=perm)
+    else:
+        authorizer.add_anonymous(options.directory, perm=perm)
+
     handler = FTPHandler
     handler.authorizer = authorizer
     handler.masquerade_address = options.nat_address
     handler.passive_ports = passive_ports
+
     ftpd = FTPServer((options.interface, options.port), FTPHandler)
+    # On Windows specify a timeout for the underlying select() so
+    # that the server can be interrupted with CTRL + C.
     try:
-        ftpd.serve_forever()
+        ftpd.serve_forever(timeout=2 if os.name == 'nt' else None)
     finally:
         ftpd.close_all()
 
+
 if __name__ == '__main__':
     main()
